using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using Terrasoft.Nui.ServiceModel.DataContract;
using Terrasoft.Core.Entities;
using System.Web.Script.Serialization;
namespace DataServiceInsertExample
{
    class Program
    {
        // Основной Uri приложения bpm'online. Необходимо заменить на пользовательский.
        private const string baseUri = @"http://example.bpmonline.com";
        // Строка запроса к методу Login сервиса AuthService.svc.
        private const string authServiceUri = baseUri + @"/ServiceModel/AuthService.svc/Login";
        // Строка пути запроса InsertQuery.
        private const string insertQueryUri = baseUri + @"/0/DataService/json/reply/InsertQuery";
        // Cookie аутентификации bpm'online.
        private static CookieContainer AuthCookie = new CookieContainer();
        // Метод выполняет аутентификацию пользователя.
        // Параметры:
        // userName — имя пользователя bpm'online,
        // userPassword — пароль пользователя bpm'online.
        private static bool TryLogin(string userName, string userPassword)
        {
            // Создание экземпляра запроса к сервису аутентификации.
            var authRequest = HttpWebRequest.Create(authServiceUri) as HttpWebRequest;
            // Определение метода HTTP запроса.
            authRequest.Method = "POST";
            // Определение типа контента запроса.
            authRequest.ContentType = "application/json";
            // Включение использования cookie в запросе.
            authRequest.CookieContainer = AuthCookie;
            // Помещение в тело запроса учетной информации пользователя.
            using (var requesrStream = authRequest.GetRequestStream())
            {
                using (var writer = new StreamWriter(requesrStream))
                {
                    writer.Write(@"{
                        ""UserName"":""" + userName + @""",
                        ""UserPassword"":""" + userPassword + @"""
                    }");
                }
            }
            // Получение ответа от сервера. Если аутентификация проходит успешно, в свойство AuthCookie будут 
            // помещены cookie, которые могут быть использованы для последующих запросов.
            using (var response = (HttpWebResponse)authRequest.GetResponse())
            {
                if (AuthCookie.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }
        // Главный метод приложения.
        static void Main(string[] args)
        {
            //Если выполнение аутентификации неудачно, приложение завершает работу.
            if (!TryLogin("Пользователь 1", "Пользователь 1"))
            {
                Console.WriteLine("Ошибка аутентификации!");
                return;
            }
            // Экземпляр класса запроса.
            var insertQuery = new InsertQuery()
            {
                // Название корневой схемы.
                RootSchemaName = "Contact",
                // Коллекция добавляемых значений колонок.
                ColumnValues = new ColumnValues()
            };
            // Экземпляр класса выражения запроса к схеме объекта.
            // Предназначен для конфигурирования колонки [ФИО].
            var columnExpressionName = new ColumnExpression
            {
                // Тип выражения запроса к схеме объекта — параметр.
                ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                // Параметр выражения запроса.
                Parameter = new Parameter
                {
                    // Значение параметра.
                    Value = "Иванов Иван Иванович",
                    // Тип данных параметра — строка.
                    DataValueType = DataValueType.Text
                }
            };
            // Экземпляр класса выражения запроса к схеме объекта.
            // Предназначен для конфигурирования колонки [Рабочий телефон].
            var columnExpressionPhone = new ColumnExpression
            {
                ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                Parameter = new Parameter
                {
                    Value = "+12 345 678 00 00",
                    DataValueType = DataValueType.Text
                }
            };
            // Экземпляр класса выражения запроса к схеме объекта.
            // Предназначен для конфигурирования колонки [Должность].
            var columnExpressionJob = new ColumnExpression
            {
                ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                Parameter = new Parameter
                {
                    // GUID записи "Разработчик" системного справочника [Должность].
                    // Необходимо заменить на GUID записи в приложении bpm'online пользователя.
                    Value = "11D68189-CED6-DF11-9B2A-001D60E938C6",
                    // Тип данных параметра — уникальный идентификатор.
                    DataValueType = DataValueType.Guid
                }
            };
            // Переопределение типа коллекции колонок запроса.
            insertQuery.ColumnValues.Items = new Dictionary<string, ColumnExpression>();
            // Добавление выражений запроса в коллекцию добавляемых колонок.
            // Колонка [ФИО].
            insertQuery.ColumnValues.Items.Add("Name", columnExpressionName);
            // Колонка [Рабочий телефон].
            insertQuery.ColumnValues.Items.Add("Phone", columnExpressionPhone);
            // Колонка [Должность].
            insertQuery.ColumnValues.Items.Add("Job", columnExpressionJob);
            // Сериализация экземпляра класса запроса на добавление в JSON-строку.
            var json = new JavaScriptSerializer().Serialize(insertQuery);
            // Преобразование строки JSON-объекта в массив байтов.
            byte[] jsonArray = Encoding.UTF8.GetBytes(json);
            // Создание экземпляра HTTP-запроса.
            var insertRequest = HttpWebRequest.Create(insertQueryUri) as HttpWebRequest;
            // Определение метода запроса.
            insertRequest.Method = "POST";
            // Определение типа содержимого запроса.
            insertRequest.ContentType = "application/json";
            // Добавление полученных ранее аутентификационных cookie в запрос на получение данных.
            insertRequest.CookieContainer = AuthCookie;
            // Set the ContentLength property of the WebRequest.
            insertRequest.ContentLength = jsonArray.Length;
            // Помещение в содержимое запроса JSON-объекта.
            using (var requestStream = insertRequest.GetRequestStream())
            {
                requestStream.Write(jsonArray, 0, jsonArray.Length);
            }
            // Выполнение HTTP-запроса и получение ответа от сервера.
            using (var response = (HttpWebResponse)insertRequest.GetResponse())
            {
                // Вывод ответа в консоль.
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            // Задержка выполнения приложения.
            Console.ReadLine();
        }
    }
}
